import React, { Component, PropTypes } from 'react';
import d3 from 'd3';
import _ from 'lodash';

/*
 * things => array of things to be represented by bars (individual rectangles)
 * valFunc => func(thing, i) returning a number that gives relative height of bar
 *
 * optional functions to call if a bar is hovered over:
 *    highlight, endHighlight => func(thing,passthrough,i,mouseEvent)
 * optional function to determine if a bar should be highlighted:
 *    isHighlighted => func(thing,passthrough,i)
 * 
 * sortBy => func(thing) should return value to determine bar order
 *           defaults to bar value
 *          
 */
class SparkBarsChart extends Component {
  shouldComponentUpdate(nextProps, nextState) {
    return this.props.things != nextProps.things ||
           this.props.selected != nextProps.selected ||
           this.props.highlighted != nextProps.highlighted;
  }
  render() {
    const {passthrough, width, height, barLabel,
        isHighlighted, highlight, endHighlight,
        orientation} = this.props;
    const valFunc = this.props.valFunc || (d=>d);
    const things = _.sortBy(this.props.things, this.props.sortBy||(d=>d));
    var ext = d3.extent(things.map((thing,i)=>valFunc(thing,i)));
    var dumbExt = [0, ext[1]];
    var offsetScale = d3.scale.linear()
                        .domain(dumbExt)
                    .   range([0, height]);
    var barSize = 1 / things.length;
    var bars = [];
    var self = this;
    things
        .forEach(function(thing, i) {
          bars.push(
            <SparkBarsBar
                passthrough={passthrough}
                thing={thing}
                barLabel={barLabel}
                valFunc={valFunc}
                orientation={orientation}
                isHighlighted={isHighlighted}
                highlight={highlight}
                endHighlight={endHighlight}
                chartHeight={self.props.height - (barLabel ? 14 : 0)}
                chartWidth={width}
                barSize={barSize} 
                offsetScale={offsetScale}
                i={i}
                key={i}
                />);
        });
    return (<div>
              <svg width={width} height={height}>
                {bars}
              </svg>
            </div>);
  }
}
SparkBarsChart.propTypes = {
  things: PropTypes.array.isRequired, 
  width: PropTypes.number.isRequired,
  height: PropTypes.number.isRequired,
};
export default SparkBarsChart;

class SparkBarsBar extends Component {
    render() {
      let {passthrough, thing, barLabel, valFunc, orientation, 
        i, offsetScale, chartHeight, chartWidth, barSize, highlight, 
        isHighlighted, endHighlight} = this.props;
      highlight = highlight || _.noop;
      isHighlighted = isHighlighted || _.noop;
      endHighlight = endHighlight || _.noop;
      let highlighted = isHighlighted(thing,passthrough,i)
      let label = '';
      if (orientation !== 'vertical') {
        let height = offsetScale(valFunc(thing, i));
        let y = chartHeight - height;
        if (barLabel) {
          const txt = barLabel(thing, i);
          label = <text y={chartHeight + 14} x={4}
                        style={{fontFamily:'Verdana',fontSize:11,
                                fontWeight:'bold', fill:'darkgray'
                        }}
                  >{txt}</text>;
        }
        return (
          <g transform={`translate(${barSize * chartWidth * i})`}>
            <rect
                    style={barStyles['background' + !!highlighted]}
                    width={barSize * chartWidth}
                    height={chartHeight} 
                    onMouseOver={evt=>highlight(thing,passthrough,i, evt)}
                    onMouseOut={evt=>endHighlight(thing,passthrough,i, evt)}
            />
            <rect y={chartHeight - height} 
                    style={barStyles['normal' + !!highlighted]}
                    width={barSize * chartWidth}
                    height={height} 
                    onMouseOver={evt=>highlight(thing,passthrough,i, evt)}
                    onMouseOut={evt=>endHighlight(thing,passthrough,i, evt)}
            />
            {label}
          </g>
        );
      } else {
        if (barLabel) {
          const txt = barLabel(thing, i);
          label = <text y={14} x={4}
                        style={{fontFamily:'Verdana',fontSize:11,
                                fontWeight:'bold', fill:'black'
                        }}
                  >{txt}</text>;
        }
        let barWidth = offsetScale(valFunc(thing, i));
        return (
          <g transform={`translate(0,${barSize * chartHeight * i})`}>
            <rect
                    style={barStyles['background' + !!highlighted]}
                    width={chartWidth} 
                    height={barSize * chartHeight} 
                    onMouseOver={evt=>highlight(thing,passthrough,i, evt)}
                    onMouseOut={evt=>endHighlight(thing,passthrough,i, evt)}
            />
            <rect x={0}
                    style={barStyles['normal' + !!highlighted]}
                    width={barWidth}
                    height={barSize * chartHeight} 
                    onMouseOver={evt=>highlight(thing,passthrough,i, evt)}
                    onMouseOut={evt=>endHighlight(thing,passthrough,i, evt)}
            />
            {label}
          </g>
        );
      }
    }
};
function barStyle(type, highlighted) {
  let opacities = {
    background: .2,
    normal: 1,
  };
  let colors = {
    background: 'steelblue',
    normal: 'steelblue',
  };
  return {
    fill: colors[type],
    strokeWidth: 1,
    stroke: highlighted ? 'steelblue' : 'white',
    opacity: opacities[type] / (highlighted ? 1 : 2)
  };
}
const barStyles = 
  _.chain([true,false])
   .map( highlighted => 
          _.map( ['normal', 'background'], type => 
                [type + highlighted, barStyle(type, highlighted)]
               )
       )
   .flatten().object().value();
